package com.jpattern.orm.session.jdbctemplate;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.List;

import org.springframework.jdbc.core.BatchPreparedStatementSetter;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.jdbc.support.KeyHolder;

import com.jpattern.orm.exception.OrmException;
import com.jpattern.orm.session.GeneratedKeyReader;
import com.jpattern.orm.session.PreparedStatementSetter;
import com.jpattern.orm.session.ResultSetReader;
import com.jpattern.orm.session.SqlPerformerStrategy;

/**
 * 
 * @author Francesco Cina
 *
 * 02/lug/2011
 * 
 * ISqlExecutor implementation using JdbcTemplate as backend
 */
public class JdbcTemplateSqlPerformerStrategy extends SqlPerformerStrategy {

	private JdbcTemplateSessionProvider sessionProvider;

	public JdbcTemplateSqlPerformerStrategy(final JdbcTemplateSessionProvider sessionProvider) {
		this.sessionProvider = sessionProvider;
	}

	@Override
	public void execute(final String sql, final int timeout) throws OrmException {
		getLogger().debug("Execute query: [{}]", sql); //$NON-NLS-1$
		try {
			getJdbcTemplate().setQueryTimeout(timeout);
			getJdbcTemplate().execute(sql);
		} catch (final Exception e) {
			throw JdbcTemplateExceptionTranslator.doTranslate(e);
		}
	}

	@Override
	public <T> T query(final String sql, final int timeout, final int maxRows, final PreparedStatementSetter pss, final ResultSetReader<T> rse)	throws OrmException {
		getLogger().debug("Execute query: [{}]", sql); //$NON-NLS-1$
		try {
			getJdbcTemplate().setMaxRows(maxRows);
			getJdbcTemplate().setQueryTimeout(timeout);
			return getJdbcTemplate().query(sql, new org.springframework.jdbc.core.PreparedStatementSetter() {
				@Override
				public void setValues(final PreparedStatement ps) throws SQLException {
					pss.set(ps);
				}
			}, new ResultSetReaderWrapper<T>(rse) );
		} catch (final Exception e) {
			throw JdbcTemplateExceptionTranslator.doTranslate(e);
		}
	}

	@Override
	public int update(final String sql, final int timeout, final PreparedStatementSetter pss) throws OrmException {
		getLogger().debug("Execute query: [{}]", sql); //$NON-NLS-1$
		try {
			getJdbcTemplate().setQueryTimeout(timeout);
			return getJdbcTemplate().update(sql, new org.springframework.jdbc.core.PreparedStatementSetter() {
				@Override
				public void setValues(final PreparedStatement ps) throws SQLException {
					pss.set(ps);
				}
			});
		} catch (final Exception e) {
			throw JdbcTemplateExceptionTranslator.doTranslate(e);
		}
	}

	@Override
	public int update(final String sql, final int timeout, final GeneratedKeyReader generatedKeyReader, final PreparedStatementSetter pss) throws OrmException {
		getLogger().debug("Execute query: [{}]", sql); //$NON-NLS-1$
		try {
			final org.springframework.jdbc.core.PreparedStatementCreator psc = new org.springframework.jdbc.core.PreparedStatementCreator() {
				@Override
				public PreparedStatement createPreparedStatement(final Connection con) throws SQLException {
					PreparedStatement ps = null;
					ps = con.prepareStatement( sql , generatedKeyReader.generatedColumnNames());
					pss.set(ps);
					return ps;
				}
			};

			final KeyHolder keyHolder = new GeneratedKeyHolder();
			getJdbcTemplate().setQueryTimeout(timeout);
			final int result = getJdbcTemplate().update(psc, keyHolder);
			generatedKeyReader.read(new GeneratorKeyResultSet(keyHolder));
			return result;
		} catch (final Exception e) {
			throw JdbcTemplateExceptionTranslator.doTranslate(e);
		}
	}

	@Override
	public int[] batchUpdate(final List<String> sqls, final int timeout) throws OrmException {
		try {
			getJdbcTemplate().setQueryTimeout(timeout);
			return getJdbcTemplate().batchUpdate(sqls.toArray(new String[0]));
		} catch (final Exception e) {
			throw JdbcTemplateExceptionTranslator.doTranslate(e);
		}
	}

	@Override
	public int[] batchUpdate(final String sql, final List<Object[]> args, final int timeout) throws OrmException {
		getLogger().debug("Execute query: [{}]", sql); //$NON-NLS-1$
		try {
			final BatchPreparedStatementSetter bpss = new BatchPreparedStatementSetter() {

				@Override
				public void setValues(final PreparedStatement ps, final int i) throws SQLException {
					int count = 0;
					for ( final Object object : args.get(i) ) {
						ps.setObject(++count, object);
					}
				}

				@Override
				public int getBatchSize() {
					return args.size();
				}
			};
			getJdbcTemplate().setQueryTimeout(timeout);
			return getJdbcTemplate().batchUpdate(sql, bpss);
		} catch (final Exception e) {
			throw JdbcTemplateExceptionTranslator.doTranslate(e);
		}
	}

	@Override
	public int[] batchUpdate(final String sql, final com.jpattern.orm.session.BatchPreparedStatementSetter psc, final int timeout) throws OrmException {
		getLogger().debug("Execute query: [{}]", sql); //$NON-NLS-1$
		try {
			final BatchPreparedStatementSetter bpss = new BatchPreparedStatementSetter() {
				@Override
				public void setValues(final PreparedStatement ps, final int i) throws SQLException {
					psc.set(ps, i);
				}
				@Override
				public int getBatchSize() {
					return psc.getBatchSize();
				}
			};
			getJdbcTemplate().setQueryTimeout(timeout);
			return getJdbcTemplate().batchUpdate(sql, bpss);
		} catch (final Exception e) {
			throw JdbcTemplateExceptionTranslator.doTranslate(e);
		}
	}

	/**
	 * @return the jdbcTemplate
	 */
	public JdbcTemplate getJdbcTemplate() {
		return sessionProvider.getJdbcTemplate();
	}

}
